# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------
'''
--------------------------------------------------------------------------
GUI
--------------------------------------------------------------------------
'''

from loge.gui.main_window import MainWindow
from loge.gui.browser import Browser
from loge.gui.main_menu import MainMenuBar
from loge.gui.main_toolbar import MainToolBar, TimerToolBar

class Gui():

    def __init__(self, app_name, app_version):
        self.Core = None
        #---
        self.main_window = MainWindow(app_name, app_version)
        # -- Menubar --
        self.menu_bar = MainMenuBar(self.main_window)
        self.menu_bar.add_menu_items(self.main_window.actions)
        self.main_window.setMenuBar(self.menu_bar)
        # -- Toolbars --
        # -- MainToolbar
        self.toolbar = MainToolBar(self.main_window)
        self.toolbar.add_toolbar_items(self.main_window.actions)
        self.main_window.addToolBar(self.toolbar)
        # -- TimerToolbar
        self.toolbar_timer = TimerToolBar(self.main_window)
        self.toolbar_timer.add_toolbar_items()
        self.toolbar_timer.setVisible(False)
        self.main_window.addToolBar(self.toolbar_timer)
        # -- Text Browser as Central Widget --
        self.browser = Browser()
        self.main_window.setCentralWidget(self.browser)
        self.browser.setOpenLinks(False)
        self.browser.anchorClicked.connect(self.on_anchor_clicked)
        # -- Statusbar --
        self.status_bar = self.main_window.statusBar()
    
    def connect_to_core(self, Core):
        self.Core = Core
        self._set_actions_slots()
        self._set_watch_slot()
        self._set_timer_slot()
        #---
        Core.Gui = self
        #---
        self.Core.startpage()

    def _set_actions_slots(self):
        self.main_window.set_action_slot('file_new', self.Core.file_new)
        self.main_window.set_action_slot('file_open', self.Core.file_open)
        self.main_window.set_action_slot('file_openreadonly', self.Core.file_openreadonly)
        self.main_window.set_action_slot('file_save', self.Core.file_save)
        self.main_window.set_action_slot('file_saveas', self.Core.file_saveas)
        self.main_window.set_action_slot('file_edit', self.Core.file_edit)
        self.main_window.set_action_slot('reload_script_file', self.Core.reload_script_file)
        self.main_window.set_action_slot('print', self.Core.file_print)
        self.main_window.set_action_slot('show_source', self.Core.show_python_source)
        self.main_window.set_action_slot('show_html', self.Core.show_html)
        self.main_window.set_action_slot('show_markdown', self.Core.show_markdown)
        self.main_window.set_action_slot('show_loge', self.Core.show_loge)
        self.main_window.set_action_slot('preview_markdown', self.Core.PreviewMarkdown)
        self.main_window.set_action_slot('save_markdown', self.Core.SaveMarkdown)
        self.main_window.set_action_slot('syntax', self.Core.show_syntax)
        self.main_window.set_action_slot('floatprecision', self.Core.floatprecision)
        self.main_window.set_action_slot('help', self.Core.help)
        self.main_window.set_action_slot('about', self.Core.about)
        self.main_window.set_action_slot('tutorial', self.Core.tutorial)

    def _set_watch_slot(self):
        """
        Sets a slot to an action of toolbar's watch script checkbox
        """
        self.toolbar.watch_check_box.clicked.connect(self.Core.watcher_clicked)

    def _set_timer_slot(self):
        """
        Sets a slot to an action of Timer button of Timer toolbar
        """
        self.toolbar_timer.timerButton.clicked.connect(self.Core.TimerButtonClicked) 

    def browser_reload(self,content):
        scroll_value = self.browser.verticalScrollBar().value()
        self.set_browser_content(content)
        self.browser.verticalScrollBar().setValue(scroll_value)

    def set_browser_content(self, content):
        self.browser.clear()
        self.browser.setHtml(content)

    def get_browser_document(self):
        return self.browser.document()

    def show(self):
        self.main_window.show()

    def get_app_main_title(self):
        return self.main_window.main_title

    def get_app_title(self):
        return self.main_window.windowTitle()

    def set_app_title(self,title):
        self.main_window.setWindowTitle(title)

    def on_anchor_clicked(self,url):
        link = str(url.toString())
        line_id = link.split(';')[0]
        setvalues = link.split(';')[1]
        index = link.split(';')[2]
        tmp = self.Core.Script.code_oryginal
        self.Core.Script.editCode(line_id, setvalues, index)
        if self.Core.refresh():
            pass
        else:
            self.Core.Script.code_oryginal = tmp

    def closeEvent(self, event):
        self.Core.Shell.close_shell()
        event.accept()


# -*- coding: utf-8 -*-
#-----------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#-----------------------------------------------------------------

"""
This module takes enables starting Loge via either "python -m loge" or "python path/to/loge"
"""

import sys
import os

from PyQt5.QtWidgets import QApplication

"""
If loge package not instaled we need to add parent directory to sys.path
"""
try:
    import loge
except ImportError:
    _this_dir = os.path.abspath(os.path.dirname(__file__))
    sys.path.insert(0, os.path.split(_this_dir)[0])

from loge.gui.Gui import Gui
from loge.core.Core import Core
from loge.memos import appinfo

def main():
    app = QApplication(sys.argv)
    CORE = Core()
    WINDOW = Gui(appinfo.name, appinfo.version)
    #---
    WINDOW.connect_to_core(CORE)
    #---
    WINDOW.main_window.closeEvent = WINDOW.closeEvent
    WINDOW.show()
    sys.exit(app.exec_())

if __name__ == '__main__':
    main()


# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------

'''
---------------------------------------------------------------------
Extra module used for code refactoring.
This is a temporary container for procedures, classes etc.
that are moved from other modules during code refactoring.
If needed they should be refactored and should be moved
to more suitable module or class.
---------------------------------------------------------------------
'''

import os
import re

import mistune

def abspath(relpath=None):
    _this_dir = os.path.dirname(os.path.abspath(__file__))
    _loge_dir = os.path.split(_this_dir)[0]
    if relpath:
        return os.path.join(_loge_dir, relpath)
    else:
        return _loge_dir
APP_PATH = abspath()

def get_html_from_markdown(markdown='#*test*'):
    html = mistune.markdown(markdown)
    return html

def get_html_from_markdownfile(markdown_abspath):
    #it is hotfix for windows problem with image display in memos
    markdown_abspath = markdown_abspath.replace("\\",'/')
    #---
    memo_page = open(markdown_abspath, 'r').read()
    #---replaceing path to images inside markdown
    img_dir =  os.path.split(markdown_abspath)[0]
    markdown = re.sub(  r'!\[(.+)\]\((.+)\)', 
                        r"![\1]({0}/\2)".format(img_dir), 
                        memo_page)
    #---
    html_page = get_html_from_markdown(markdown)
    return html_page

def get_html_from_memo(markdown_memo='x_startpage.md'):
    meno_abspath = abspath(os.path.join('memos', markdown_memo))
    html_page = get_html_from_markdownfile(meno_abspath)
    return html_page

def get_html_from_code(code='a=1'):
    markdown_page = "````\n" + code + "\n````"
    html_page = get_html_from_markdown(markdown_page)
    return html_page


#! ##Working with SVG graphic


#! ###You can load SVG file ...

#%img python.svg

#! ###or render SVG python string

svgsyntax='''
<svg height="55" width="55">
    <circle cx="30" cy="30" r="20"
    stroke="black" stroke-width="1" fill="tan" /> 
</svg>
'''
svgsyntax #%svg

#! so this is the way to get parametric drawing

r = 60 #<< - circle radius value
xs = 48 #<< - center x
ys = 66 #<< - center y

svgsyntax='''
<svg height="150" width="200">
    <circle cx="{1}" cy="{2}"
    r="{0}" stroke="black" stroke-width="1" fill="tan" />
    <text x="{1}" y="{2}" fill="black"
    font-size="15">circle {0} radius </text>  
</svg>
'''.format(r, xs, ys)

#! for this parameters we have

svgsyntax #%svg

#!
'''
###To make parameterise easer you can use `svgwrite` package
'''
import svgwrite

a = 90 #<< - rec a dimension
b = 89 #<< - rec b dimension

svg_document = svgwrite.Drawing(size = (200, 100))
svg_document.add(svg_document.rect(insert = (0, 0), size = (a, b),
                                   stroke_width = "1",stroke = "black",fill = "tan"))
svg_document.add(svg_document.text("Rectangle size" + str(a)+ 'x' +str(b) ,
                                   insert = (a/2-20, b/2)))

svg_document #%svg




# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------

import os
import sys
import re
import copy
import tempfile

from PyQt5.QtWidgets import QFileDialog

from loge.core.core_utils import get_html_from_markdown


import loge.core.shell_super_functions as super_functions



class Shell():
    def __init__(self):
        self.report_markdown = ''
        self.report_html = ''
        #---
        self.savedir = os.path.dirname(__file__)
        #---
        self.Script = None
        #----
        self.tmpdir = None
        self.create_tempdir()
        #----
        self.tempfile_nameprefix = 'tmp_loge_'
        #----
        self.float_display_precison = 3

    #------------------------
        
    def assign_code(self, CodeObiect):
        self.Script = CodeObiect

    #------------------------
    
    def create_tempdir(self):
        if not self.tmpdir:
            dirpath = tempfile.mkdtemp()
            dirname = os.path.basename(dirpath)
            new_dirname = 'loge_' + dirname
            new_dirpath = dirpath.replace(dirname, new_dirname)
            os.rename(dirpath, new_dirpath)
            self.tmpdir = new_dirpath

    def get_tmp_file_path(self, filename):
        filename = self.tempfile_nameprefix + filename # adding prefix to name
        tmp_file_path = os.path.join(self.tmpdir, filename)
        return tmp_file_path

    #------------------------

    def run_oryginal(self):
        exec (self.Script.code_oryginal in globals(), locals())

    def run_parsed(self):
        self.report_markdown = ''
        self.report_html = ''
        self._id = 1
        #-----------------------------------------------------------------
        # Here are functions needed in namespace where parsed code is run
        #-----------------------------------------------------------------
        super_functions.r_shell = self
        super_functions.variables = vars()        
        #--------
        r_comment = super_functions.r_comment
        r_seepywarning = super_functions.r_seepywarning
        r_mathcomment = super_functions.r_mathcomment
        r_adj = super_functions.r_adj
        r_img = super_functions.r_img
        r_plt = super_functions.r_plt           
        r_pil = super_functions.r_pil
        r_tex = super_functions.r_tex
        r_codetex = super_functions.r_codetex
        codeformat = super_functions.codeformat
        r_svg = super_functions.r_svg
        vars_formated = super_functions.vars_formated
        #---Adding current script dir to python PATH list
        #---(user will be able to import modules from dir where his seepy script is stored)
        script_dir = os.path.dirname(self.Script.script_path)
        sys.path.append(script_dir) 
        #-----------------------------------------------------------------
        #---------- Here the code_parsed is finally executed -------------
        #-----------------------------------------------------------------
        exec(self.Script.code_parsed)
        #-----------------------------------------------------------------
        #-----------------------------------------------------------------
        #---so the report_markdown has been created-----------------------
        #---and mistune is used to get report_html from report_markdown
        self.report_html = get_html_from_markdown(self.report_markdown)
        #-----------------------------------------------------------------
        self._id = 0
        #---Deleting current script dir from python PATH list
        sys.path = list(set(sys.path)) # first deleting duplicates
        sys.path.remove(script_dir) # and finaly deleting script dir 

    #------------------------
    
    def __del__ (self):
        if self.tmpdir:
            self.close_shell()
        
    def close_shell (self):
        self.delete_tmpfile(deleteall=True)
        os.removedirs(self.tmpdir)
        self.tmpdir = None
            
    def delete_tmpfile(self, deleteall=False):
        if deleteall:
            for content in os.listdir(self.tmpdir):
                os.remove(self.tmpdir + '/' + content)
        else:
            for content in os.listdir(self.tmpdir):
                if self.tempfile_nameprefix in content :
                    os.remove(self.tmpdir + '/' + content)
            
    #------------------------
    
    def save_report_markdown(self, savedir = os.path.dirname(__file__), initfilename = 'new.md'):
        #---asking for file path
        filename = QFileDialog.getSaveFileName(caption = 'Save as Markdown document',
                                                directory = self.savedir + '/' + initfilename,
                                                filter = "Markdown document (*.md)")[0]
        filename = str(filename)
        #---
        if not filename == '':
            self.savedir = os.path.dirname(filename)
            md_file = open(filename, "w")
            md_file.write(self.report_markdown)
            md_file.close()


#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------


# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------

import subprocess
import idlelib
import traceback
import os
#import codecs
from PyQt5 import QtCore
from PyQt5.QtWidgets import QApplication, QMessageBox, QDialog, QInputDialog
from PyQt5.QtPrintSupport import QPrintDialog
from PyQt5.QtGui import QCursor

from loge.core.Script import Script
from loge.core.Shell import Shell

from loge.core.core_utils import abspath, get_html_from_memo, get_html_from_code, get_html_from_markdownfile

class Core():
    
    def __init__(self):
        self.Script = Script()
        self.Shell = Shell()
        self.Gui = None
        #---
        self.watcher = None
        self.timer = None
        #---
        self.set_watcher()
        self.set_timer()
        self.Shell.assign_code(self.Script)

    def set_watcher(self):
        self.watcher = QtCore.QFileSystemWatcher()
        self.watcher.fileChanged.connect(self.script_changed)
        self.Script.set_watcher(self.watcher)

    def set_timer(self):
        # -- Timer --
        self.timer = QtCore.QTimer()
        self.timer.timeout.connect(self.TimerAction)

    def watcher_clicked(self,checked):
        if checked:
            self.Script.activate_watcher()
        else:
            self.Script.deactivate_watcher()

    def script_changed(self,file):
        self.Script.reloadcode()
        self.refresh()

    #------------------------ 

    def refresh(self, waitcursor = True):
        successruned = False
        #some OS updating file first save 0 bits than save right size
        #because of that set scrollBar doesn't work
        #we don't want reload browser if file size is zero 
        file_info = QtCore.QFileInfo(self.Script.script_path)
        if 0 == file_info.size():
            return successruned
        message = None
        error_trace = None
        if waitcursor:
            QApplication.setOverrideCursor(QCursor(QtCore.Qt.WaitCursor))
        try:
            self.Script.parse()
            self.Shell.run_parsed()
            self.Gui.browser_reload(self.Shell.report_html)
            successruned = True
        except Exception as e:
            message = 'Some problem - ' + str(e)
            error_trace = str(traceback.format_exc())
        finally:
            QApplication.restoreOverrideCursor()
        
        if not successruned and message:
            QMessageBox.information(None, message, error_trace)
        #---
        apptitle = ' - ' + os.path.basename(self.Script.script_path)
        if self.Script.saveLock:
            apptitle += '  (read only)'
        apptitle = self.Gui.get_app_main_title() + apptitle
        self.Gui.set_app_title(apptitle)
        #---
        return successruned

    def reload_script_file(self):
        if self.Script.script_path :
            self.Script.reloadcode()
            self.refresh()
        else:
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    #------------------------ 

    def file_new(self):
        template_path = abspath('templates/x_newtemplate.py')
        self.Script.newFile(template_path, 'Save new script as', 'newScript.py')
        self.refresh()
            
    def file_open(self, file_path = None, saveLock = False):
        self.Shell.delete_tmpfile() #Cleaning Loge tmp files
        self.Script.openFile(file_path, saveLock)
        self.refresh()
        #--Timer
        self.TimerFromCode()

    def file_openreadonly(self, file_path = None):
        self.file_open(file_path = None, saveLock = True)

    def file_save(self):
        if self.Script.script_path :
            if self.Script.savecode():
                QMessageBox.information(None, 'Info', 'Saved to '+ self.Script.script_path)
        else:
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    def file_saveas(self):
        if self.Script.script_path :
            self.Script.saveAs()
            self.refresh()
        else:
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    def file_edit(self):
        if self.Script.saveLock:
            QMessageBox.information(None, 'Info', 'File open read only. Use save as option')
            return False
        if self.Script.script_path :
            subprocess.Popen(['python', '-m', 'idlelib.idle', self.Script.script_path]) 
        else:
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    #------------------------ 

    def show_python_source(self):
        if self.Script.script_path :
            html_page = get_html_from_code(self.Script.code_oryginal)
            self.Gui.set_browser_content(html_page)
        else:
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    def show_html(self):
        if self.Script.script_path :
            html_page = get_html_from_code(self.Shell.report_html)
            self.Gui.set_browser_content(html_page)
        else:
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    def show_markdown(self):
        if self.Script.script_path:
            html_page = get_html_from_code(self.Shell.report_markdown)
            self.Gui.set_browser_content(html_page)
        else:
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    def show_loge(self):
        if self.Script.script_path :
            self.Gui.set_browser_content(self.Shell.report_html)
        else:
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    def show_syntax(self):
        html_page = get_html_from_memo('x_syntax.md')
        self.Gui.set_browser_content(html_page)

    #------------------------
    def PreviewMarkdown(self):
        #---asking for file path
        filename = self.Script.open_markdown()
        if not filename == '':
            #---
            #show_markdown_file(filename)
            html_page = get_html_from_markdownfile(filename)
            self.Gui.set_browser_content(html_page)

    def SaveMarkdown(self):
        if self.Script.script_path :
            initname = os.path.basename(self.Script.script_path).replace('.py', '.md')
            self.Shell.save_report_markdown(self.Script.savedir, initname)
        else:
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    #------------------------ 
            
    def startpage(self):
        html_page = get_html_from_memo('x_startpage.md')
        self.Gui.set_browser_content(html_page)

    def help(self):
        html_page = get_html_from_memo('x_help.md')
        self.Gui.set_browser_content(html_page)

    def about(self):
        html_page = get_html_from_memo('x_about.md')
        self.Gui.set_browser_content(html_page)

    #------------------------ 

    def file_print(self):
        dialog = QPrintDialog()
        if dialog.exec_() == QDialog.Accepted:
            self.Gui.get_browser_document().print_(dialog.printer())
    
    #------------------------ 

    def tutorial(self):
        tutorial_path = abspath('memos/x_tutorial.py')
        savedir = self.Script.savedir #coping current savedir
        #--opening oryginal tutorial
        self.Shell.delete_tmpfile()
        self.Script.openFile(tutorial_path)
        #---saving temporary tutorial copy
        tutorial_tmppath = os.path.join(self.Shell.tmpdir, 'tmp_tutorial.py')
        self.Script.saveAs(tutorial_tmppath)
        #---opening temporary tutorial copy
        self.Shell.delete_tmpfile()
        self.Script.openFile(tutorial_tmppath)
        self.Script.savedir = savedir #back to previouse savedir
        self.refresh()
        self.file_edit()

    #------------------------ 

    def floatprecision(self):
        if self.Script.script_path :
            #---asking for precision as int number
            value = QInputDialog.getInt(    None, 
                                            'Float display precysion', 'Set the precison:',
                                            value = self.Shell.float_display_precison,
                                            min = 1, max = 9, step = 1)[0]
            #---
            self.Shell.float_display_precison = value
            self.refresh()
        else:
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    #-----------------------

    def TimerButtonClicked(self):
        if self.Script.script_path:
            if self.Gui.toolbar_timer.timerButton.isChecked():
                self.TimerStart()
            if not self.Gui.toolbar_timer.timerButton.isChecked():
                self.TimerStop()
        else:
            self.TimerStop()
            QMessageBox.information(None, 'Info', 'Please create or open script first')

    def TimerAction(self):
        if self.Script.script_path:
            if self.Gui.toolbar_timer.timerButton.isChecked():
                self.Gui.toolbar_timer.timerPulse.setChecked(not(self.Gui.toolbar_timer.timerPulse.isChecked()))
                self.refresh(False)

    #---------------------
    def TimerStart(self):
        timespace = int(self.Gui.toolbar_timer.timerSpinBox.value() * 1000)
        self.timer.start(timespace)
        self.Gui.toolbar_timer.timerButton.setChecked(True)
        self.Gui.toolbar_timer.timerSpinBox.setEnabled(False)
        self.Gui.toolbar_timer.timerPulse.setEnabled(True)
        self.refresh(False)
                
    def TimerStop(self):
        self.timer.stop()
        self.Gui.toolbar_timer.timerButton.setChecked(False)
        self.Gui.toolbar_timer.timerSpinBox.setEnabled(True)
        self.Gui.toolbar_timer.timerPulse.setChecked(False)
        self.Gui.toolbar_timer.timerPulse.setEnabled(False)

    def TimerFromCode(self):
        parameters = self.Script.getTimerParameters()
        if parameters:
            self.Gui.toolbar_timer.timerButton.setChecked(parameters[0])
            self.Gui.toolbar_timer.timerSpinBox.setValue(parameters[1])
            if parameters[0]:
                self.TimerStart()
        else:
            self.TimerStop()
            self.Gui.toolbar_timer.timerButton.setChecked(False)
            self.Gui.toolbar_timer.timerSpinBox.setValue(2.0)



#! #*Loge tutorial*

#%img x_start.png

#!
'''
Please look at Loge report window and your tutorial *.py file editor that has just run.
Please try to change the scrip in editor and use save<Ctrl+s> to see how report has changed.
But first check does the watch script option in toolbar is active.
'''

#! ------------------------------------------------------------

#! Let's write some python script to do some calculations.

a = 12
b = 25
c = a + b

print(a)


#!
'''
You can run this script as normal *.py script - run it by <F5> in script editor.
You have seen the `a` value in shell output. But you can see it in your Loge report ...
'''

a #! - here it is

#! Let's change a value 

a = 30 #! - here is changed value


#! ------------------------------------------------------------

#!
'''
You can call variable values in this comments.
So, we have %(a)s , %(b)s and %(c)s.

And the other way is val_a and var_a
'''

#! ------------------------------------------------------------

#!
'''
You can still run this script as
normal *.py script - run it by <F5> in script editor.
As you can guess python engine does't see the Loge syntax.
'''

#! ------------------------------------------------------------

#!
'''
Loge comments use Markdown.

#Title
##Title
###Title

*some text*

**some text**

* Title
* Title2

More about Markdown you can see here
www.daringfireball.net/projects/markdown/

Here is Markdown tutorial
www.markdowntutorial.com/
'''

#! ------------------------------------------------------------

#! You can see python code from your script in Loge report

#%code
r = 120
import math
math.pi
area = math.pi * r ** 2
#%

#! for one line code

area = math.pi * r ** 2 #%code


area #!- here is what we get

#! ------------------------------------------------------------

#! Go to Script>Float precission in menu to change float precission.

#! ------------------------------------------------------------

#!
'''
Lets show image in your report,
the image file must be storef in the same directory where you script is.
'''

#%img x_python.png

#! ... here our Python is.

#! ------------------------------------------------------------
#!
'''
The Python Imaging Library adds image processing capabilities to your Python interpreter.
You can display image from PIL.Image instance. Here is ismply example, it will display resized 
and rotated image.
'''
import os
from PIL import Image
imagefilepath = '/home/.../someimage.jpg' #<<<< Click to select image path (jpg,png..)  -
if os.path.isfile(imagefilepath):
    im = Image.open(imagefilepath) 
    #! Resizeing
    im = im.resize((200,200)) #%pil
    #! Rotating
    angle = 40 #<<<< Rotate angle - 
    im2 = im.rotate(angle) #%pil

#! Here you can find more about PILLOW www.pillow.readthedocs.io/en/3.4.x/index.html

#! ------------------------------------------------------------

#! What about plotting?? You can use Matplotlib pyplot - here is some example

import matplotlib.pyplot as plt
import numpy as np
t = np.arange(-1.0, 2.0, 0.01)
s1 = np.cos(9*np.pi*t) + 3 * t ** 2
plt.plot(t, s1)
plt #%plt
plt.clf()

#! ------------------------------------------------------------

#! If you need to publish some mathematical formula LaTex syntax can be used

#! You can write some LaTex as comment
#%tex s(t) = \mathcal{A}\mathrm{sin}(2 \omega t)

#! or if the LaTex fomula is in you python code
pi = 3.14
r = 40
Area = pi * r ** 2 #%tex

#! or if the LaTex fomula is defined as python string
LaTexString = '\lim_{x \to \infty} \exp(-x) = 0'
LaTexString #%stringtex

#! ------------------------------------------------------------

#!
'''
You can change python variables in report, so finally you don't need to edit
script source to change input data.
'''

a = 120 #! - this is not interactive variable in your report
b = 66 #<< - this is interactive variable in your report - click to change it
#! Those values are var_a and var_b.

#! Other display effects are possible
b = 66 #<< your comment
b = 66 #<<< your comment
b = 66 #<<<< your comment

#! If your variable is equal some list element, you can select list element interactively:
somelist = [1, 2, 3, 4]
variable = somelist[2] #<< - select variable value

#! or with othet display effects
variable = somelist[2] #<< - select variable value
variable = somelist[1] #<<< - select variable value
variable = somelist[1] #<<<< select variable value

#! ####There are some special cases available

#! If you variable value is True or False then interactive CheckBox will be displayed on report when  or used. When you click CheckBox the value will change to opposite bool value.
boolvarialbe = True #<<< - click me
#! We have val_boolvarialbe
boolvarialbe = True #<<<< Click me -
#! We have val_boolvarialbe
boolvarialbe = True #<< - click me
#! We have val_boolvarialbe

#! If your variable name has 'filepath' or 'dirpath' in name then file or directory browse dialog will be open after clicked.

image_filepath = '/home/image.ipg' #<<< - click me to get new file path
#! Your file path is val_image_filepath

search_dirpath = '/home' #<<<< Click me to get new dir path -
#! Your file path is val_search_dirpath

#! *Examples of use*

#! ####*Example 1*
car_list = ['volvo', 'toyota', 'saab', 'fiat']
your_car = car_list[0] #<<<< Select your car -
carcolor = 'black' 

getcolor = False #<<<Browse val_your_car color if not val_carcolor
if getcolor:
    car_colors = [ carcolor, 'pink', 'red', 'magenta']
    carcolor = car_colors[0] #<<<< >>Select your color -
#! Your car is val_carcolor  val_your_car .

#! ####*Example 2*
material_list = ['steel', 'concrete', 'plastic', 'wood']
material = material_list[0] #<<<< Material is - 
#! The %(material)s will be used to make something.

#! ####*Example 3*
temperature_range = list(range(10,30,1))
docontrol = False #<<< Temperature control state
if docontrol:
    room_temperature = temperature_range[17] #<<<< Select temperature - 
    #! Temperature %(room_temperature)s Celsius degree selected
else :
    pass
    
#! ------------------------------------------------------------

#! What about drawing in report ?? You can create drawing using SVG syntax.

#! Python string with SVG syntax can be rendered.

svgsyntax='''
<svg height="55" width="55">
    <circle cx="30" cy="30" r="20"
    stroke="black" stroke-width="1" fill="tan" /> 
</svg>
'''
svgsyntax #%svg

#! so this is the way to get parametric drawing

r = 50 #<< - circle radius value
xs = 120 #<< - center x
ys = 70 #<< - center y
svgsyntax='''
<svg height="150" width="200">
    <circle cx="{1}" cy="{2}"
    r="{0}" stroke="black" stroke-width="1" fill="tan" />
    <text x="{1}" y="{2}" fill="black"
    font-size="15">circle {0} radius </text>  
</svg>
'''.format(r, xs, ys)
#! for this parameters we have
svgsyntax #%svg

#!To make parametrise easer you can use `svgwrite` package

import svgwrite
a = 60 #<< - rec a dimension
b = 100 #<< - rec b dimension
svg_document = svgwrite.Drawing(size = (200, 100))
svg_document.add(svg_document.rect(insert = (0, 0), size = (a, b),
                                   stroke_width = "1",stroke = "black",fill = "tan"))
svg_document.add(svg_document.text("Rectangle size" + str(a)+ 'x' +str(b) ,
                                   insert = (a/2-20, b/2)))
svg_document #%svg 

#! ------------------------------------------------------------

#! Mathematical expressions and equations
#! You can add to Loge report expressions and equations with optional comment 
a = 1
b = 3
c = 3*a + 4*b #%requ - your comment
c = 3*a + 4*b #%requ
c = 3*a + 4*b #%equ - your comment
c = 3*a + 4*b #%equ
3*a + 4*b #%requ - your comment
3*a + 4*b #%requ
3*a + 4*b #%equ - your comment
3*a + 4*b #%equ

#! Python code is formated to look more natural

import math
math.pi #%equ
math.sin(1) #%equ
2**2 #%equ

#! When you are working with SI units using Unum package

from unum import units as u
5 * u.m + 10 * u.mm #%requ
(5 * u.m + 10 * u.mm).asUnit(u.km) #%equ

#! ------------------------------------------------------------

#! ##Go to Info>Syntax help to get know abaut all available  Loge syntax.



# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------

import os
import re
import shutil

import codecs

from PyQt5.QtWidgets import QFileDialog, QInputDialog, QMessageBox, QLineEdit

from loge.memos import appinfo
from loge.core.core_utils import APP_PATH
from loge.core import script_parsing as script_parsing

class Script():
    
    def __init__(self):
        #---
        self._script_path = ''
        self._script_path_observers = []
        self.savedir = APP_PATH
        #---
        self.code_oryginal = ''
        self.code_parsed = ''
        #---
        self.stamptext = appinfo.report_stamp
        #--
        self.saveLock = False
        #--
        self.watcher = None
        self._watcher_active = False

    #------------------------
    
    @property 
    def script_path(self):
        return self._script_path

    @script_path.setter
    def script_path(self,value):
        self._script_path = value
        if self._watcher_active: #if checkbox checked (watcher active) activate it for new path
            self.activate_watcher()

    #------------------------

    def set_watcher(self, watcher):
        self.watcher = watcher

    def activate_watcher(self):
        #print('watcher script path: '+self.script_path)
        if self.watcher and self.script_path != '':
            #print('dodaje sciezke: '+self.script_path)
            self.deactivate_watcher() #clear current watcher's paths
            if self.watcher.addPath(self.script_path):
                #print('sciezka dodana '+ str(self.watcher.files()))
                pass
            else:
                #print('sciezka nie dodana')
                pass
        self._watcher_active = True

    def deactivate_watcher(self):
        if self.watcher and self.script_path != '':
            #clear watcher paths
            if self.watcher.files():
                self.watcher.removePaths(self.watcher.files())
        self._watcher_active = False

    #------------------------

    def parse(self):
        script = self.code_oryginal
        #--parsing extra loge command syntax
        script = script_parsing.parse(script)
        #--report end stamp
        script += "\nr_comment('>>*----- %s -----*')"%self.stamptext
        #--saving
        self.code_parsed = script 

    #------------------------

    def editCode(self, lineID = 'id1', setvalues = None, index = None):
        if setvalues == 'None':
            setvalues = None
        #---
        script = self.code_oryginal
        #---
        script = re.sub(r'#(<{2,})', r"#\1_idx_", script)
        no = 1  
        while re.search(r"#<{2,}_idx_", script):
            script = script.replace(r'<_idx_', r"<_id%s_" % no, 1)
            no += 1 
        #---OPTION 1 Selectind one form list if list
        if setvalues :
            setvalues = re.search(r'[[](.+)[]]', setvalues).group(1)
            setvalues = setvalues.replace(" ", "")
            setvalues = setvalues.replace("'", "")
            setvalues = setvalues.split(',')
            #---
            expresion = re.search(r'(\w+)\s*=\s*(\w+)\s*[[](\d+)[]]\s*#<{2,}_%s_'%lineID, script)
            variable = expresion.group(1)
            listindex = int(expresion.group(3))
            #---asking for new value from choice list
            value_selected = QInputDialog.getItem(None, 'Set new value', variable +'=', setvalues, listindex, False)[0]
            #---
            if value_selected:
                index_selected = setvalues.index(value_selected)
            else:
                index_selected = listindex
            #---
            script = re.sub(    r'(\w+)\s*=\s*(.+)[[]\w+[]]\s*#(<{2,})_%s_'%lineID,
                                r'\1 = \2[%s] #\3_%s_'%(index_selected, lineID),
                                script  )
        #---OPTION 2 Geting new variable value   
        else:
            expresion = re.search(r'(\w+)\s*=\s*(.+)\s*#<{2,}_%s_'%lineID, script)
            variable = expresion.group(1)
            oldvalue = expresion.group(2)
            oldvalue = oldvalue.rstrip() #for now this hotfix delete whitespace from the end of oldvalue string
            newvalue = None
            #---asking for new value
            if ('filepath' in variable): # if variable look like filepath
                directory = '/'
                try:
                    if os.path.isdir(os.path.dirname(eval(oldvalue))):
                        directory = os.path.dirname(eval(oldvalue))
                except:
                    pass
                askmsg = "Select new filepath for '%s' variable" %variable
                filename = QFileDialog.getOpenFileName(caption=askmsg, directory=directory)[0]
                if filename == '':
                    newvalue = oldvalue
                else:
                    newvalue = "'%s'"%str(filename)
            elif ('dirpath' in variable): # if variable look like dirpath
                directory = '/'
                try:
                    if os.path.isdir(eval(oldvalue)):
                        directory = os.path.dirname(eval(oldvalue))
                except:
                    pass
                dirname = QFileDialog.getExistingDirectory(caption = 'Select directory', directory = directory)
                if dirname == '':
                    newvalue = oldvalue
                else:                
                    newvalue = "'%s'"%str(dirname)
            elif (oldvalue.strip() in ['True', 'False']): # if variable has bool value
                bool_oldvalue = eval(oldvalue)
                newvalue = str(not(bool_oldvalue))
            else: # other cases
                input_result = QInputDialog.getText(None, 'Set new value',variable +'=', QLineEdit.Normal,oldvalue)
                if input_result[1]:
                    newvalue = input_result[0]

            #---
            if newvalue:
                script = re.sub(r'(\w+)\s*=\s*(.+)\s*#(<{2,})_%s_'%lineID,
                                r'\1 = %s #\3_%s_'%(newvalue, lineID),
                                script  )
        #---
        script = re.sub(r"#(<{2,})_id(\d+)_", r'#\1', script)
        #---
        self.code_oryginal = script

    #------------------------        

    def getTimerParameters(self):
        is_active = False
        timespace = None
        #---
        script = self.code_oryginal
        timer = re.search(r"#%timer[ ]*(\d*)[ ]*ON", script)
        if timer:
            is_active = True
            timespace  = float(timer.group(1)) / 1000
            return is_active, timespace
        timer = re.search(r"#%timer[ ]*(\d*)[ ]*OFF", script)
        if timer:
            is_active = False
            timespace  = float(timer.group(1)) / 1000
            return is_active, timespace
        return None
        
    #------------------------ 

    def savecode(self):
        if self.saveLock:
            QMessageBox.information(None, 'Info', 'File open read only. Use save as option')
            return False
        else:
            file = codecs.open(self.script_path,'r+', 'utf-8')
            file.write(self.code_oryginal)
            file.close()
            return True

    def openFile(self, file_path=None, saveLock = False):
        #---asking for file path if not given
        if file_path:
            filename =  file_path
        else:
            filename = QFileDialog.getOpenFileName(caption = 'Open script',
                                                    directory = self.savedir,
                                                    filter = "Python script (*.py)")
            filename = str(filename[0])
        #---
        if not filename == '':
            self.savedir = os.path.dirname(filename)
            self.script_path = filename
            file = codecs.open(self.script_path, 'r', 'utf-8')
            self.code_oryginal = file.read()    
            file.close()      
            self.parse()
            self.saveLock = saveLock
        else:
            pass
    
    def open_markdown(self):
        filename = QFileDialog.getOpenFileName(caption = 'Open Markdown document',
                                                directory = self.savedir,
                                                filter = "Markdown document (*.md)")
        filename = str(filename[0])
        return filename

    def newFile(self, template_path, info='Save as', initfilename='your_script', filedirectory = None):
        #---asking for file path
        if filedirectory is None:
            filename,ffilter = QFileDialog.getSaveFileName(caption = info,
                                                    directory = self.savedir + '/' + initfilename,
                                                    filter = "Python script (*.py)")
        else:
            filename = filedirectory + '/' + initfilename
        #---
        if not filename == '':
            new_template = codecs.open(template_path,'r', 'utf-8').read()
            self.savedir = os.path.dirname(filename)
            text_file = codecs.open(filename, 'w', 'utf-8')
            text_file.write(new_template)
            text_file.close()
            self.script_path = filename
            self.reloadcode()
            self.parse()
            self.saveLock = False

    def saveAs(self, savepath = None):
        newname = 'Copy_' + os.path.basename(self.script_path)
        #---asking for file path
        initdir = self.savedir + '/' + newname
        if self.saveLock:
            initdir = '/' + newname
        if savepath:
            filename = savepath
        else:
            filename = QFileDialog.getSaveFileName(caption = 'Save as',
                                                        directory = initdir,
                                                        filter = "Python script (*.py)")[0]
            filename = str(filename)
        #---
        new_file = codecs.open(filename,'w', 'utf-8')
        new_file.write(self.code_oryginal)
        new_file.close()
        #---coping files linked inside code_oryginal
        src_dir = os.path.dirname(self.script_path)
        dst_dir = os.path.dirname(filename)
        copied_dependences = []
        for fname in os.listdir(src_dir):   
            if  fname in self.code_oryginal:
                scr_file = os.path.join(src_dir, fname)
                dst_file = os.path.join(dst_dir, fname)
                if not scr_file == dst_file:
                    if os.path.isfile(scr_file) and ('.' in scr_file):
                        shutil.copyfile(scr_file, dst_file)
                        copied_dependences.append(fname)
        #---
        self.script_path = filename
        self.savedir = os.path.dirname(filename)
        self.reloadcode()
        self.parse()
        self.saveLock = False
        #---info massage
        infotext = 'Saved to '+ filename + '\n'
        if not copied_dependences == []:
            infotext += 'Copied_dependences: \n' 
            for i in copied_dependences:
                infotext += str(i) + '\n' 
        QMessageBox.information(None, 'Info', infotext)

    def reloadcode(self):
        file = codecs.open(self.script_path,'r', 'utf-8')
        self.code_oryginal = file.read()    
        file.close()


# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------

"""
---------------------------------------------------------------------
A module containing utilities.
---------------------------------------------------------------------
"""

from pkg_resources import resource_string

from PyQt5.QtWidgets import QAction
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtCore import QCoreApplication

TR_GUI_CONTEXT = 'Gui'

def translate(context,msg):
    return QCoreApplication.translate(context,msg)

def get_icon(icon_name):
    """
    Creates an icon from file.
    
    Args:
        icon_name (str): the name of an icon file

    Retruns:
        QIcon: An icon object
    """
    pixmap = QPixmap()
    pixmap.loadFromData(resource_string(__name__, '/icons/{}'.format(icon_name)))
    icon = QIcon(pixmap)
    return icon

def create_action(icon_name,act_name,shortcut,tip,parent):
    """
    Creates an action object.
    
    Args:
        icon_name (str):    a name of an icon file
        act_name (str):     an action name
        shortcut (str):     a shortcut string or QKeySequence standart shortcut
        tip (str):          a tip text
        parent (obj):       a parent object instance

    Retruns:
        QAction: An icon object
    """
    action = QAction(act_name,parent)
    if icon_name is not None:
        action.setIcon(get_icon(icon_name))
    if shortcut is not None:
        action.setShortcut(shortcut)
    if tip is not None:
        action.setToolTip(tip)
        action.setStatusTip(tip)
    return action


'''
--------------------------------------------------------------------------
Copyright (C) 2016 Lukasz Laba <lukaszlab@o2.pl>

This file is part of SeePy.
SeePy is a structural engineering design Python package.
http://struthon.org/

SeePy is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

SeePy is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
--------------------------------------------------------------------------
'''


# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------


import re
import copy
import os

try:
    import svgwrite
except ImportError:
    pass
try:
    import matplotlib.pyplot as plt
except ImportError:
    pass
    
#----------------------------------------------

r_shell = None
variables = vars()

def vars_formated():
    out = copy.copy(variables)
    for key in out:
        if type(out[key]) is float:
            out[key] = round(out[key], r_shell.float_display_precison)
    return out

#----------------------------------------------

def r_comment(object):
    r_shell.report_markdown += str(object) + '\n\n'
    
def r_seepywarning(warning):
    r_comment('>>*!!! SeePyWarning - %s !!!*'%warning)

def r_mathcomment(object):
    comment_string = str(object)
    comment_string_formated = codeformat(comment_string)
    r_comment(comment_string_formated)
            
def r_adj(text = 'text', link = 'link', comment = 'somecomment', mode = 1, code = ''):
    islist = re.search(r'(\w+)\s*=\s*(\w+)\s*[[](\d+)[]]\s*', code)
    setvalues = None
    index = None
    #---changing True False display on report
    if text == 'True':
        text = '☑'
    if text == 'False':
        text = '☐'
    #---
    if islist:
        variable = islist.group(2)
        index = int(islist.group(3))
        setvalues = ('%(' + str(variable) + ')s') % vars_formated()
    if mode == 1:
        href='[{0}]({1};{3};{4}) {2}'.format(text, link, comment, setvalues, index)
    if mode == 2:
        href='{2} [{0}]({1};{3};{4})'.format(text, link, comment, setvalues, index)
    r_shell.report_markdown += href +'\n\n'
    
def r_img(imagename):
    image_path = os.path.dirname(r_shell.Script.script_path) + '/' + imagename
    r_shell.report_markdown += '![Alt text](%s)\n\n' % image_path

def r_plt(pltObject):
    try:
        name = str(r_shell._id) + '.png'
        image_path = r_shell.get_tmp_file_path(name)
        pltObject.savefig(image_path, dpi=(60))
        r_shell.report_markdown += '![Alt text](%s)\n\n' % image_path
        r_shell._id += 1
    except Exception as e :
        r_seepywarning('Matplotlib plt image save failure - %s' %str(e))
        
def r_pil(PilImageObject):
    try:
        name = str(r_shell._id)+ '.png'
        image_path = r_shell.get_tmp_file_path(name)
        PilImageObject.save(image_path)
        r_shell.report_markdown += '![Alt text](%s)\n\n' % image_path
        r_shell._id += 1
    except Exception as e :
        r_seepywarning('Pillow image save failure - %s' %str(e))
        
def r_tex(string):
    plt.figure(frameon=False)
    plt.axes(frameon=0)
    if string[0] != '$' and string[-1] != '$':
        string = '$' + string + '$'
    plt.text(0.0, 0.0, string, fontsize=600)
    plt.xticks(())
    plt.yticks(())
    plt.tight_layout()
    name = str(r_shell._id) + '.png'
    image_path = r_shell.get_tmp_file_path(name)
    plt.savefig(image_path, bbox_inches='tight', dpi=(2))
    plt.close()
    r_shell.report_markdown += '![Alt text](%s)\n\n' % image_path
    r_shell._id += 1

def r_codetex(string):
    r_tex(codeformat(string))

def r_svg(svgObject):
    name = str(r_shell._id) + '.svg'
    image_path = r_shell.get_tmp_file_path(name)
    if type(svgObject) in [str]:
        svg_file = open(image_path, "w")
        svg_file.write(svgObject)
        r_shell.report_markdown += '![Alt text](%s)\n\n' % image_path
        svg_file.close()
        r_shell._id += 1
    elif type(svgObject) is svgwrite.drawing.Drawing:
        svg_file = open(image_path, "w")
        svg_file.write(svgObject.tostring())
        r_shell.report_markdown += '![Alt text](%s)\n\n' % image_path
        svg_file.close()
        r_shell._id += 1
    else:
        r_seepywarning('Unknown SVG format given')

#----------

def codeformat(string):
    #changing 3**3 to 3^2
    string = string.replace('**', '^')
    #changing math.sin(1) to sin(1)
    string = string.replace('math.', '')
    #changing 3 * u.mm to 3[mm] - usable when Unum SI units system used in script
    string = re.sub(    r'\s*\*\s*u.(\w+)',
                        r'[\1]',
                        string  )
    #changing (3*u.mm + 3*u.m).asUnit(u.mm) to 3*u.mm + 3*u.m - usable when Unum SI units system used in script
    string = re.sub(    r'\((.+)\).asUnit\((.+)\)',
                        r'\1',
                        string  )            
    return string


import math

#!##*Circle Area calculation*

#%img CircleArea_fig_1.png

#! For input data

r = 77 #<< - circle radius

#! from formula that everyone know we have


Area = math.pi * r**2 #%requ

#! ###So, the area of circle with var_r is val_Area.







# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------
'''
---------------------------------------------------------------------
Toolbar module of a main window.
---------------------------------------------------------------------
'''

from PyQt5.QtWidgets import QToolBar, QCheckBox, QPushButton, QDoubleSpinBox, QRadioButton

from loge.gui.gui_utils import translate, TR_GUI_CONTEXT

class MainToolBar(QToolBar):

    def __init__(self,parent):
        super().__init__(translate(TR_GUI_CONTEXT,'Main'),parent)

    def add_toolbar_items(self,actions):
        if not actions:
            raise ValueError('An attribute "actions" dictionary can not be empty')
        if not isinstance(actions, dict):
            raise TypeError('An attribute "actions" should be a dictionary type')

        self.addAction(actions['file_new'])
        self.addAction(actions['file_open'])
        self.addAction(actions['file_save'])
        self.addSeparator()
        self.addAction(actions['reload_script_file'])
        self.watch_check_box = QCheckBox(translate(TR_GUI_CONTEXT,'watch script'))
        self.watch_check_box.setStatusTip(translate(TR_GUI_CONTEXT,'If active report will be reloaded after script file change'))
        self.addWidget(self.watch_check_box)
        self.addSeparator()
        self.addAction(actions['file_edit'])
        self.addSeparator()
        self.addAction(actions['print'])
        self.addAction(actions['save_markdown'])
        self.addSeparator()
        self.addAction(actions['show_source'])
        self.addAction(actions['show_html'])
        self.addAction(actions['show_markdown'])
        self.addAction(actions['show_loge'])
        self.addSeparator()
        self.addAction(actions['preview_markdown'])
        self.addSeparator()
        self.addAction(actions['syntax'])


class TimerToolBar(QToolBar):

    def __init__(self,parent):
        super().__init__(translate(TR_GUI_CONTEXT,'Timer'),parent)

    def add_toolbar_items(self):
        self.timerButton = QPushButton(translate(TR_GUI_CONTEXT,'Timer'), self)
        self.timerButton.setCheckable(True)
        self.timerButton.setStatusTip(translate(TR_GUI_CONTEXT,"If pushed timer active"))
        
        self.timerSpinBox = QDoubleSpinBox(self)
        self.timerSpinBox.setStatusTip(translate(TR_GUI_CONTEXT,"Time space in seconds"))
        self.timerSpinBox.setMinimum(0.1)
        self.timerSpinBox.setSingleStep(0.1)
        self.timerSpinBox.setValue(2.0)
        self.timerPulse = QRadioButton(self)
        self.timerPulse.setEnabled(False)

        self.addWidget(self.timerButton)
        self.addWidget(self.timerSpinBox)
        self.addWidget(self.timerPulse)



# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------
"""
---------------------------------------------------------------------
The module containing a main application window class.
---------------------------------------------------------------------
"""

from PyQt5.QtWidgets import QDesktopWidget, QMainWindow, QStyle
from PyQt5.QtGui import QIcon, QKeySequence
from PyQt5.QtCore import Qt, QSize

from loge.gui.gui_utils import get_icon, create_action
from loge.gui.gui_utils import translate, TR_GUI_CONTEXT

# a scale of application window width to cover the % of the whole screen available space
_SW_WIDTH = 0.53 #53% because of width an image in start page
# a scale of application window height to cover the % of the whole screen available space
_SW_HEIGHT = 0.7
# application window minimum width in pixels
_MIN_W_WIDTH = 100
# application window minimum height in pixels
_MIN_W_HEIGHT = 100
# application logo icon
_APP_ICON = "logo.png"


class MainWindow(QMainWindow):

    def __init__(self,app_name,app_version):
        super().__init__()
        self.actions = {} #actions dictionary besides QWidget.actions() list
        self._init_main_window()
        self.main_title = app_name + ' (' + app_version+')'
        self.setWindowTitle(self.main_title)
        self._set_actions()


    def _init_main_window(self):
        # -- Window settings --
        ag = QDesktopWidget().availableGeometry()
        window_size = ag.size()
        app_width = _SW_WIDTH * window_size.width()
        app_height = _SW_HEIGHT * window_size.height()
        window_size.scale(app_width,app_height,Qt.IgnoreAspectRatio)
        self.setGeometry(
        QStyle.alignedRect(Qt.LeftToRight,
            Qt.AlignCenter,
            window_size,
            ag
        ))
        self.setMinimumSize(_MIN_W_WIDTH,_MIN_W_HEIGHT)
        self.setWindowIcon(get_icon(_APP_ICON))

    def _set_actions(self):
        self.actions['file_new'] = create_action('new.png','New',QKeySequence.New
                            ,'Create new *.py script from template',self)

        self.actions['file_open'] = create_action('open.png','Open',QKeySequence.Open
                              ,'Open existing *.py script',self)
        
        self.actions['file_openreadonly'] = create_action('openreadonly.png','Open read only'
                                      ,None,'Open existing *.py script - edit and saving not aloved',self)
        
        self.actions['file_save'] = create_action('save.png','Save',QKeySequence.Save
                               ,'Save changes to script file',self)

        self.actions['file_saveas'] = create_action('saveas.png','Save as',QKeySequence.SaveAs
                                 ,'Save current script',self)

        self.actions['reload_script_file'] = create_action('reload.png','Reload',QKeySequence.Refresh
                                 ,'Reload *.py script',self)

        self.actions['file_edit'] = create_action('edit.png','Edit','Ctrl+E'
                              ,'It opens to edit current *.py script',self)

        self.actions['show_html'] = create_action('showhtml.png','Show report as HTML',None
                                  ,'It shows report as HTML code',self)

        self.actions['show_markdown'] = create_action('showmarkdown.png','Show report as Markdown',None
                                     ,'It shows report as Markdown code',self)

        self.actions['show_source'] = create_action('showsource.png','Show python source',None
                                    ,'It shows python source code',self)

        self.actions['show_loge'] = create_action('showreport.png','Back to Loge report',None
                                   ,'Back to Loge report',self)

        self.actions['preview_markdown'] = create_action('previewmarkdown.png','Preview some Markdown',None
                                   ,'You can preview some Markdown document - it not change watched *.py script',self)

        self.actions['save_markdown'] = create_action('savemarkdown.png','Save report as Markdown file',None
                                   ,'Save raport as Markdown file',self)
#
#        #savemarkdownAction = QAction(QIcon(get_icon_path("savemarkdown.png")),"Save report as Markdown file",self)
#        #savemarkdownAction.setStatusTip("Save report as Markdown file")
#        #savemarkdownAction.triggered.connect(self.SaveMarkdown)
#
#        #printAction = QAction(QIcon(get_icon_path("print.png")),"Print document",self)
#        #printAction.setStatusTip("Print document")
#        #printAction.setShortcut("Ctrl+P")
#        #printAction.triggered.connect(self.Print)
        self.actions['print'] = create_action('print.png',translate(TR_GUI_CONTEXT,'Print document'),QKeySequence.Print
                               ,translate(TR_GUI_CONTEXT,'Print document'),self)

        self.actions['help'] = create_action('help.png',translate(TR_GUI_CONTEXT,'Help'),QKeySequence.HelpContents
                              ,translate(TR_GUI_CONTEXT,'Help information'),self)

        self.actions['about'] = create_action('about.png',translate(TR_GUI_CONTEXT,'About')
                               ,None,translate(TR_GUI_CONTEXT,'Loge project information'),self)

        self.actions['floatprecision'] = create_action(None,translate(TR_GUI_CONTEXT,'Float precision'),None
                                        ,translate(TR_GUI_CONTEXT,'Set float display precision'),self)

        self.actions['syntax'] = create_action('syntax.png',translate(TR_GUI_CONTEXT,'Syntax help'),'F2'
                                ,translate(TR_GUI_CONTEXT,'Show Loge syntax help'),self)

        self.actions['tutorial'] = create_action('tutorial.png',translate(TR_GUI_CONTEXT,'Tutorial'),None
                                  ,translate(TR_GUI_CONTEXT,'Open tutorial script'),self)        

    def set_action_slot(self,action_name,slot):
        """
        Sets a slot to an action.

        Args:
            action_name (str): the name of an action
            slot: a function that is called in response to a signal
        """
        self.actions[action_name].triggered.connect(slot)



# -*- coding: utf-8 -*-

from glob import glob
from distutils.core import setup
from setuptools import find_packages
from os.path import splitext, basename

setup(
    name='loge',
    version='0.1.8',
    description='Easy and fast dynamic report generation with Python3',
    long_description = open("README.rst").read(),
    author='Lukasz Laba',
    author_email='lukaszlab@o2.pl',
    url='https://loge.readthedocs.io',
    license = 'GNU General Public License (GPL)',
    keywords = 'notebook ,script, report',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: X11 Applications :: Qt',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering',
        ],
    python_requires='>=3.5, <4',
    package_dir={'': 'src'},  # tell distutils packages are under src
    packages=find_packages('src'),  # include all packages under src
    include_package_data=True, # include everything in source control
    # ...but exclude README.txt from all packages
    exclude_package_data={'': ['README.txt']},
    install_requires=['pyqt5>=5.6','mistune>=0.7.4'],        
    entry_points={
        'console_scripts':['loge = loge.__main__:main']
        }
    )


#! ##Working with Matplotlib and LaTex
import matplotlib.pyplot as plt
import numpy as np

t = np.arange(-1.0, 2.0, 0.01)

#! ##Fig. 1
s1 = np.cos(9*np.pi*t) + 3 * t ** 2 #%tex
plt.figure(1)
plt.plot(t, s1) #%plt
plt.clf()

#! ##Fig. 2
s2 = np.sin(18*np.pi*t**2) #%tex
plt.figure(2)
plt.plot(t, s2)
plt #%plt
plt.clf()

#! ##Fig. 3
s3 = t**3 #%tex
plt.figure(3)
plt.plot(t, s3)
plt #%plt
plt.clf()

#! ##Fig. 4 - All in one
plt.figure(4)
plt.plot(t, s1)
plt.plot(t, s2)
plt.plot(t, s3)
plt #%plt
plt.clf()

#! You can use LaTex syntax

#%tex A_s = f_d/A_s + 5
#%tex s(t) = \mathcal{A}\mathrm{sin}(2 \omega t)

a = 23
#%tex f(x) = %(a)s * y + 4^3

#%tex 26*i*2 + 4/2^6 kjkjk





# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------

'''
---------------------------------------------------------------------
Menu bar module of a main window.
---------------------------------------------------------------------
'''

from PyQt5.QtWidgets import QMenuBar

from loge.gui.gui_utils import translate, TR_GUI_CONTEXT

class MainMenuBar(QMenuBar):

    def __init__(self,parent):
        super().__init__(parent)
        self._init_menu_bar()

    def _init_menu_bar(self):
        self.m_file = self.addMenu(translate(TR_GUI_CONTEXT,"&File"))
        self.m_script = self.addMenu(translate(TR_GUI_CONTEXT,"&Script"))
        self.m_help = self.addMenu(translate(TR_GUI_CONTEXT,"&Help"))


    def add_menu_items(self,actions):
        if not actions:
            raise ValueError('An attribute "actions" dictionary can not be empty')
        if not isinstance(actions, dict):
            raise TypeError('An attribute "actions" should be a dictionary type')

        self.m_file.addAction(actions['file_new'])
        self.m_file.addAction(actions['file_open'])
        self.m_file.addAction(actions['file_openreadonly'])
        self.m_file.addSeparator()
        self.m_file.addAction(actions['file_save'])
        self.m_file.addAction(actions['file_saveas'])
        self.m_file.addSeparator()
        self.m_file.addAction(actions['print'])

        self.m_script.addAction(actions['reload_script_file'])
        self.m_script.addAction(actions['file_edit'])
        self.m_script.addAction(actions['show_html'])
        self.m_script.addAction(actions['show_markdown'])
        self.m_script.addAction(actions['show_loge'])
        self.m_script.addAction(actions['floatprecision'])

        self.m_help.addAction(actions['help'])
        self.m_help.addAction(actions['syntax'])
        self.m_help.addAction(actions['tutorial'])
        self.m_help.addSeparator()
        self.m_help.addAction(actions['about'])


# -*- coding: utf-8 -*-
from strupy import units as u

#!
'''
##*Obliczenie zbrojenia w przekroju zginanym prostokątnym*

*(metoda maksymalnie uproszczona)*
'''

#! ###1.Wymiary przekroju
h = 1200 *   u.mm  #<< - wysoko
b = 400 * u.mm #<< - szerokość przekroju

#! ###2.Obciążenie
Msd = 1200 *  u.kNm #<< - moment obliczeniowy

#! ###3.Materiał
materials = [300 * u.MPa, 400 * u.MPa, 500 * u.MPa]
fyd = materials[1] #<< - stal zbrojeniowaśćą

#! ---

#%img PrzekZginany_fig_1.png

#! Ze wzoru:

As1 = (  Msd / (0.8 * h) * 1 / fyd  ).asUnit(u.mm2) #%requ

#!
'''
---
###Podsumowanie
Dla przekroju o wymiarach val_b x val_h i obciażeniu var_Msd 
potrzebne zbrojenie dołem var_As1.
'''















from PIL import Image, ImageDraw

imagefilepath = '/home/....../someimage.jpg'  #<<<< Image path -

im = None

try:
    im = Image.open(imagefilepath)
except:
    None
    #! File not find

if im:
    #!---
    doshow = False #<<<< Show loaded image
    if doshow:
        #! *Loaded image viev*
        im #%pil
    #! ---
    doresize = True #<<<< Resizing
    if doresize:
        a = 400 #<<<< >>width
        b = 400 #<<<< >>high
        im = im.resize((a,b))
    #! ---
    dorotate = False #<<<< Rotating
    if dorotate :       
        angle = 0 #<<<< >>rotate angle [deg] -
        im = im.rotate(angle)
    #! ---
    doblack = False #<<<< Black white
    if doblack :       
        im = im.convert(mode = 'L')
    #!---
    #! Finaly image looking
    im #%pil
    #!---
    savefilepath = '/home/lukaszlab/Pulpit/alamakota.jpg' #<<<< Save as -
    dosave = False #<<<< Save
    if dosave :
        im.save(savefilepath)
        #! Saved to val_savefilepath!!
        pass
    




# -*- coding: utf-8 -*-
#--------------------------------------------------------------------
# Copyright (C) 2017, the Loge development team
#
# This file is part of Loge
# Loge is distributed under the terms of GNU General Public License
# The full license can be found in 'license.txt'
# Loge development team can be found in 'development.txt'
#--------------------------------------------------------------------


import re

def parse(script=''):
    #-------------------------------------------------------------------------
    #Here the code_oryginal is changed in to code_parsed with re.sub() replace
    #-------------------------------------------------------------------------
    
    script = re.sub(r'\r\n', r'\n', script) #new line \n (Linux) vs \r\n (windows) problem
    
    #--Variable with one line comment syntax (indentation acceptable)
    script = re.sub(    r'([ \t]*)(\w+)(.+)#!(.*)',
                        r"\1\2\3 \n\1r_comment('''```\2 = %(\2)s``` \4''' % vars_formated())",
                        script  )
                        
    #--Result EQUation  with one line comment syntax, version /a = d + 4 = 5/ (indentation acceptable)
    script = re.sub(    r'([ \t]*)(\w+)\s*=\s*(.+)#%requ(.*)',
                        r"\1\2 = \3 \n\1r_mathcomment('''```\2 = \3 = %(\2)s``` \4''' % vars_formated())",
                        script  )
                        
                        
    #--EQUation with one line comment syntax, version /a = d + 4/ (indentation acceptable)
    script = re.sub(    r'([ \t]*)(\w+)\s*=\s*(.+)#%equ(.*)',
                        r"\1\2 = \3 \n\1r_mathcomment('''```\2 = \3``` \4''' % vars_formated())",
                        script  )
                        
    #--Result EQUation with one line comment syntax, version /d + 4 = 5/ (indentation acceptable)
    script = re.sub(    r'([ \t]*)(\S[^=\^\n]+)#%requ(.*)',
                        r"\1\2 \n\1r_tmp =  \2 \n\1r_mathcomment('''```\2 = %(r_tmp)s``` \3'''.format(\2) % vars_formated()) \n\1r_tmp =  None",
                        script  )
                        
    #--EQUation with one line comment syntax, version /d + 4/  (indentation acceptable)
    script = re.sub(    r'([ \t]*)(\S[^=\^\n]+)#%equ(.*)',
                        r"\1\2 \n\1r_mathcomment('''```\2``` \3''' % vars_formated())",
                        script  )
                        
    #--One line comment syntax (indentation acceptable)
    script = re.sub(    r'#![ ]*(.+)',
                        r"r_comment('''\1''' % vars_formated())",
                        script  )
    #--Multi line comment syntax (indentation not acceptable)
    script = re.sub(    r"#!(.{1})'''(.+?)'''",
                        r"r_comment('''\2''' % vars_formated())", 
                        script, flags=re.DOTALL )
    #--One line python code showing syntax (indentation not acceptable)
    script = re.sub(    r"(.+)#%code",
                        r"\1\nr_comment('''```\1```''' )",
                        script  )
    #--Multi line python code showing syntax (indentation not acceptable)
    script = re.sub(    r"#%code(.+?)#%", 
                        r"\1r_comment('''```\1```''' )",
                        script, flags=re.DOTALL )
    #--Image showing syntax (indentation acceptable)
    script = re.sub(    r'#%img (.+)',
                        r"r_img('\1')", 
                        script  )
    #--Matplotlib plt figure syntax (indentation acceptable)
    script = re.sub(    r'([ \t]*)(\w+)(.+)#%plt',
                        r"\1\2\3 \n\1r_plt(\2)",
                        script)
    #--Pillow Image syntax (indentation acceptable)
    script = re.sub(    r'([ \t]*)(\w+)(.+)#%pil',
                        r"\1\2\3 \n\1r_pil(\2)",
                        script)
    #--One line LaTex syntax comment rendering (indentation acceptable)
    script = re.sub(    r'#%tex[ ]*(.+)',
                        r"r_tex(r'\1' % vars())", 
                        script  )
    #--Rendering LaTex syntax from python string (indentation acceptable)
    script = re.sub(    r'([ \t]*)(\w+)\s*#%stringtex',
                        r"\1\2 \n\1r_tex(\2)",
                        script)
    #--One line code rendering as LaTex syntax (indentation acceptable)
    script = re.sub(    r'([ \t]*)(\w[^\n]+)#%tex',
                        r"\1\2 \n\1r_codetex(r'\2' % vars_formated())",
                        script  )
    #--Rendering SVG syntax from python string (indentation acceptable)
    script = re.sub(    r'([ \t]*)(\w+)(.+)#%svg',
                        r"\1\2\3 \n\1r_svg(\2)",
                        script)
    #--Adjustable wariable with one line comment syntax (indentation not tested yet)
    script = re.sub(r'#(<{2,})', r"#\1_idx_", script)
    no = 1  
    while re.search(r"#<{2,}_idx_", script):
        script = script.replace(r'<_idx_', r"<_id%s_" % no, 1)
        no += 1
    script = re.sub(    r'([ \t]*)(\w+)(.+)#<<_(id\d+)_(.+)', 
                        r"\1\2\3 \n\1r_adj('''```\2 = %(\2)s```''' % vars_formated(),'\4','\5' % vars_formated(), 1, '''\2\3''')",
                        script  )
    script = re.sub(    r'([ \t]*)(\w+)(.+)#<<<_(id\d+)_(.+)', 
                        r"\1\2\3 \n\1r_adj('%(\2)s' % vars_formated(),'\4','\5' % vars_formated(), 1, '''\2\3''')",
                        script  )
    script = re.sub(    r'([ \t]*)(\w+)(.+)#<<<<_(id\d+)_(.+)', 
                        r"\1\2\3 \n\1r_adj('%(\2)s' % vars_formated(),'\4','\5' % vars_formated(), 2,  '''\2\3''')",
                        script  )
                        
    #--calling variable with val_name and var_name
    script = re.sub(r'val_(\w+)', r''' ```%(\1)s``` ''', script)
    script = re.sub(r'var_(\w+)', r''' ```\1 = %(\1)s``` ''', script)
    return script